import Utils = require('../Utils');
import i = require('../Interfaces');
import _ = require('lodash')
import Path = require('path')
import Prompt = require("../Prompt");
var score: (i: string, m: string) => number = require('fuzzaldrin').score;

var descriptions: {[indexer: string]: string} = {
    admin: 'Create and administer SCCS files',
    alias: 'Define or display aliases',
    ar: 'Create and maintain library archives',
    asa: 'Interpret carriage-control characters',
    at: 'Execute commands at a later time',
    awk: 'Pattern scanning and processing language',
    basename: 'Return non-directory portion of a pathname; see also dirname',
    batch: 'Schedule commands to be executed in a batch queue',
    bc: 'Arbitrary-precision arithmetic language',
    bg: 'Run jobs in the background',
    cc: 'Compile standard C programs',
    cal: 'Print a calendar',
    cat: 'Concatenate and print files',
    cd: 'Change the working directory',
    cflow: 'Generate a C-language flowgraph',
    chgrp: 'Change the file group ownership',
    chmod: 'Change the file modes/attributes/permissions',
    chown: 'Change the file ownership',
    cksum: 'Write file checksums and sizes',
    cmp: 'Compare two files; see also diff',
    comm: 'Select or reject lines common to two files',
    command: 'Execute a simple command',
    compress: 'Compress data',
    cp: 'Copy files',
    crontab: 'Schedule periodic background work',
    csplit: 'Split files based on context',
    ctags: 'Create a tags file',
    cut: 'Cut out selected fields of each line of a file',
    cxref: 'Generate a C-language program cross-reference table',
    date: 'Display the date and time',
    dd: 'Convert and copy a file',
    delta: 'Make a delta (change) to an SCCS file',
    df: 'Report free disk space',
    diff: 'Compare two files; see also cmp',
    dirname: 'Return the directory portion of a pathname; see also basename',
    du: 'Estimate file space usage',
    echo: 'Write arguments to standard output',
    ed: 'The standard text editor',
    env: 'Set the environment for command invocation',
    ex: 'Text editor',
    expand: 'Convert tabs to spaces',
    expr: 'Evaluate arguments as an expression',
    FALSE: 'Return false value',
    fc: 'Process the command history list',
    fg: 'Run jobs in the foreground',
    file: 'Determine file type',
    find: 'Find files',
    fold: 'Filter for folding lines',
    fort77: 'FORTRAN compiler',
    fuser: 'List process IDs of all processes that have one or more files open',
    gencat: 'Generate a formatted message catalog',
    get: 'Get a version of an SCCS file',
    getconf: 'Get configuration values',
    getopts: 'Parse utility options',
    grep: 'Search text for a pattern',
    hash: 'hash database access method',
    head: 'Copy the first part of files',
    iconv: 'Codeset conversion',
    id: 'Return user identity',
    ipcrm: 'Remove a message queue, semaphore set, or shared memory segment identifier',
    ipcs: 'Report interprocess communication facilities status',
    jobs: 'Display status of jobs in the current session',
    join: 'Merges two sorted text files based on the presence of a common field',
    kill: 'Terminate or signal processes',
    lex: 'Generate programs for lexical tasks',
    link: 'Create a hard link to a file',
    ln: 'Link files',
    locale: 'Get locale-specific information',
    localedef: 'Define locale environment',
    logger: 'Log messages',
    logname: 'Return the user\'s login name',
    lp: 'Send files to a printer',
    ls: 'List directory contents',
    m4: 'Macro processor',
    mailx: 'Process messages',
    make: 'Maintain, update, and regenerate groups of programs',
    man: 'Display system documentation',
    mesg: 'Permit or deny messages',
    mkdir: 'Make directories',
    mkfifo: 'Make FIFO special files',
    more: 'Display files on a page-by-page basis',
    mv: 'Move files',
    newgrp: 'Change to a new group (functionaliy similar to sg[1])',
    nice: 'Invoke a utility with an altered nice value',
    nl: 'Line numbering filter',
    nm: 'Write the name list of an object file',
    nohup: 'Invoke a utility immune to hangups',
    od: 'Dump files in various formats',
    paste: 'Merge corresponding or subsequent lines of files',
    patch: 'Apply changes to files',
    pathchk: 'Check pathnames',
    pax: 'Portable archive interchange',
    pr: 'Print files',
    printf: 'Write formatted output',
    prs: 'Print an SCCS file',
    ps: 'Report process status',
    pwd: 'print working directory - Return working directory name',
    qalter: 'Alter batch job',
    qdel: 'Delete batch jobs',
    qhold: 'Hold batch jobs',
    qmove: 'Move batch jobs',
    qmsg: 'Send message to batch jobs',
    qrerun: 'Rerun batch jobs',
    qrls: 'Release batch jobs',
    qselect: 'Select batch jobs',
    qsig: 'Signal batch jobs',
    qstat: 'Show status of batch jobs',
    qsub: 'Submit a script',
    read: 'Read a line from standard input',
    renice: 'Set nice values of running processes',
    rm: 'Remove directory entries',
    rmdel: 'Remove a delta from an SCCS file',
    rmdir: 'Remove directories',
    sact: 'Print current SCCS file-editing activity',
    sccs: 'Front end for the SCCS subsystem',
    sed: 'Stream editor',
    sh: 'Shell, the standard command language interpreter',
    sleep: 'Suspend execution for an interval',
    sort: 'Sort, merge, or sequence check text files',
    split: 'Split files into pieces',
    strings: 'Find printable strings in files',
    strip: 'Remove unnecessary information from executable files',
    stty: 'Set the options for a terminal',
    tabs: 'Set terminal tabs',
    tail: 'Copy the last part of a file',
    talk: 'Talk to another user',
    tee: 'Duplicate the standard output',
    test: 'Evaluate expression',
    time: 'Time a simple command',
    touch: 'Change file access and modification times',
    tput: 'Change terminal characteristics',
    tr: 'Translate characters',
    TRUE: 'Return true value',
    tsort: 'Topological sort',
    tty: 'Return user\'s terminal name     ',
    type: 'Displays how a name would be interpreted if used as a command',
    ulimit: 'Set or report file size limit',
    umask: 'Get or set the file mode creation mask',
    unalias: 'Remove alias definitions',
    uname: 'Return system name',
    uncompress: 'Expand compressed data',
    unexpand: 'Convert spaces to tabs',
    unget: 'Undo a previous get of an SCCS file',
    uniq: 'Report or filter out repeated lines in a file',
    unlink: 'Call the unlink function',
    uucp: 'System-to-system copy',
    uudecode: 'Decode a binary file',
    uuencode: 'Encode a binary file',
    uustat: 'uucp status inquiry and job control',
    uux: 'Remote command execution',
    val: 'Validate SCCS files',
    vi: 'Screen-oriented (visual) display editor',
    wait: 'Await process completion',
    wc: 'Line, word and byte or character count',
    what: 'Identify SCCS files',
    who: 'Display who is on the system',
    write: 'Write to another user\'s terminal',
    xargs: 'Construct argument lists and invoke utility',
    yacc: 'Yet another compiler compiler',
    zcat: 'Expand and concatenate data'
};

class Executable implements i.AutocompletionProvider {
    private paths: Array<string> = process.env.PATH.split(Path.delimiter);
    private executables: string[] = [];

    constructor() {
        this.paths.forEach(path =>
            Utils.filesIn(path).then(files => this.executables = this.executables.concat(files))
        );
    }

    getSuggestions(prompt: Prompt) {
        return new Promise((resolve) => {
            if (prompt.getWholeCommand().length > 1) {
                return resolve([]);
            }

            var lastArgument = prompt.getLastArgument();

            var all = _.map(this.executables, (executable: string) => {
                return {
                    value: executable,
                    score: 1.5 * score(executable, lastArgument),
                    synopsis: '',
                    description: descriptions[executable],
                    type: 'executable'
                };
            });

            resolve(_(all).sortBy('score').reverse().take(10).value());
        });
    }
}

export = Executable;
